@@ -1,5 +1,5 @@
 /*
- * "$Id: cups-driverd.cxx 10070 2011-10-14 15:51:23Z mike $"
+ * "$Id: cups-driverd.cxx 10276 2012-02-13 22:48:22Z mike $"
  *
  *   PPD/driver support for CUPS.
  *
@@ -7,7 +7,7 @@
  *   created from driver information files, and dynamically generated PPD files
  *   using driver helper programs.
  *
- *   Copyright 2007-2011 by Apple Inc.
+ *   Copyright 2007-2012 by Apple Inc.
  *   Copyright 1997-2007 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
@@ -2294,17 +2294,19 @@ load_ppds(const char *d,		/* I - Actual directory */
       ppd->record.model_number = model_number;
       ppd->record.type         = type;
 
+      strlcpy(ppd->record.filename, name, sizeof(ppd->record.filename));
       strlcpy(ppd->record.name, name, sizeof(ppd->record.name));
-      strlcpy(ppd->record.make, manufacturer, sizeof(ppd->record.make));
-      strlcpy(ppd->record.make_and_model, make_model,
-              sizeof(ppd->record.make_and_model));
       strlcpy(ppd->record.languages[0], lang_version,
               sizeof(ppd->record.languages[0]));
       strlcpy(ppd->record.products[0], (char *)cupsArrayFirst(products),
               sizeof(ppd->record.products[0]));
       strlcpy(ppd->record.psversions[0], (char *)cupsArrayFirst(psversions),
               sizeof(ppd->record.psversions[0]));
+      strlcpy(ppd->record.make, manufacturer, sizeof(ppd->record.make));
+      strlcpy(ppd->record.make_and_model, make_model,
+              sizeof(ppd->record.make_and_model));
       strlcpy(ppd->record.device_id, device_id, sizeof(ppd->record.device_id));
+      strlcpy(ppd->record.scheme, "file", sizeof(ppd->record.scheme));
     }
 
    /*
@@ -2580,5 +2582,5 @@ regex_string(const char *s)		/* I - String to compare */
 
 
 /*
- * End of "$Id: cups-driverd.cxx 10070 2011-10-14 15:51:23Z mike $".
+ * End of "$Id: cups-driverd.cxx 10276 2012-02-13 22:48:22Z mike $".
  */
